use openssl::{
    bn::{BigNum, BigNumContext},
    ec::{EcGroup, EcPoint},
};

pub fn initialize_elliptic_curve(
    p_sel: &str,
    a_sel: &str,
    b_sel: &str,
    generator_x_sel: &str,
    generator_y_sel: &str,
    public_key_x_sel: &str,
    public_key_y_sel: &str,
) -> (EcGroup, EcPoint, EcPoint) {
    let mut context = BigNumContext::new().unwrap();

    let p = BigNum::from_dec_str(p_sel).unwrap();
    let a = BigNum::from_dec_str(a_sel).unwrap();
    let b = BigNum::from_dec_str(b_sel).unwrap();
    let generator_x = BigNum::from_dec_str(generator_x_sel).unwrap();
    let generator_y = BigNum::from_dec_str(generator_y_sel).unwrap();

    let public_key_x = BigNum::from_dec_str(public_key_x_sel).unwrap();
    let public_key_y = BigNum::from_dec_str(public_key_y_sel).unwrap();

    let c_curve = EcGroup::from_components(p, a, b, &mut context).unwrap();

    let mut gen_point = EcPoint::new(&c_curve).unwrap();
    let _ =
        gen_point.set_affine_coordinates_gfp(&c_curve, &generator_x, &generator_y, &mut context);

    let mut pub_point = EcPoint::new(&c_curve).unwrap();
    let _ =
        pub_point.set_affine_coordinates_gfp(&c_curve, &public_key_x, &public_key_y, &mut context);

    (c_curve, gen_point, pub_point)
}

pub fn bitmask(n: u64) -> u64 {
    (1 << n) - 1
}

pub fn next_sn_bits(field: u64, n: u32, offset: u32) -> u64 {
    (field >> offset) & ((1u64 << n) - 1)
}

pub fn by_dword(n: &[u8]) -> u32 {
    (n[0] as u32) | (n[1] as u32) << 8 | (n[2] as u32) << 16 | (n[3] as u32) << 24
}
